import sequelizeConnection from '../config'
import {DataTypes, Model, Optional} from "sequelize";

interface VehicleTypeAttributes {
    id: number;
    name: string;
    seats: number;
}
export interface VehicleTypeAttributesInput extends Optional<VehicleTypeAttributes, 'id'> {}
export interface VehicleTypeAttributesOutput extends Required<VehicleTypeAttributes> {}
class VehicleType extends Model<VehicleTypeAttributes> implements VehicleTypeAttributes {
    public id!: number;
    public name!: string;
    public seats: number;

    public readonly createdAt!: Date;
    public readonly updatedAt!: Date;
}
VehicleType.init(
    {
        id: {
            type: DataTypes.INTEGER,
            autoIncrement: true,
            primaryKey: true,
        },
        name: {
            type: new DataTypes.STRING,
            allowNull: false,
        },
        seats: {
            type: new DataTypes.INTEGER,
            allowNull: false,
        },
    },
    {
        tableName: "vehicleTypes",
        sequelize: sequelizeConnection,
    }
);

export const create = async (payload: VehicleType): Promise<VehicleTypeAttributesOutput> => {
    return await VehicleType.create(payload)
}

export const update = async (id: number, payload: Partial<VehicleTypeAttributesInput>): Promise<VehicleTypeAttributesOutput> => {
    const type = await VehicleType.findByPk(id)
    if (!type) {
        throw new Error('not found')
    }
    return await (type as VehicleType).update(payload)
}

export const getById = async (id: number): Promise<VehicleType> => {
    const type = await VehicleType.findByPk(id)
    if (!type) {
        throw new Error('not found')
    }
    return type
}

export const deleteById = async (id: number): Promise<boolean> => {
    const deletedTypeCount = await VehicleType.destroy({
        where: {id}
    })
    return !!deletedTypeCount
}

export const getAll = async (): Promise<VehicleTypeAttributesOutput[]> => {
    return VehicleType.findAll()
}
export const VehicleTypeService = {
    create, update, getById, deleteById, getAll
}

export default VehicleType
